/**
 * Copyright (c) 2016-2018 Thales Global Services S.A.S.
 * This program and the accompanying materials are made
 * available under the terms of the Eclipse Public License 2.0
 * which is available at https://www.eclipse.org/legal/epl-2.0/
 * 
 * SPDX-License-Identifier: EPL-2.0
 * 
 * Contributors:
 *   Abel Hegedus, Tamas Borbas, Balazs Grill, Peter Lunk, Daniel Segesdi (IncQuery Labs Ltd.) - initial API and implementation
 */
package org.eclipse.emf.diffmerge.patch.runtime.modelaccess;

import org.eclipse.emf.common.util.EList;
import org.eclipse.emf.diffmerge.patch.runtime.modelaccess.AbstractEMFModleAccess;
import org.eclipse.emf.ecore.EClass;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.EStructuralFeature;
import org.eclipse.emf.ecore.util.EcoreUtil;

@SuppressWarnings("all")
public class SimpleReflectiveEMFModelAccess extends AbstractEMFModleAccess {
  @Override
  protected EObject doCreate(final EClass type) {
    final EObject eObject = type.getEPackage().getEFactoryInstance().create(type);
    return eObject;
  }
  
  @Override
  protected void doAdd(final EObject container, final EStructuralFeature feature, final Object element) {
    boolean _isMany = feature.isMany();
    if (_isMany) {
      Object _eGet = container.eGet(feature);
      final EList featureValueList = ((EList) _eGet);
      featureValueList.add(element);
    } else {
      container.eSet(feature, element);
    }
  }
  
  @Override
  protected void doAdd(final EObject container, final EStructuralFeature feature, final Object element, final int index) {
    Object _eGet = container.eGet(feature);
    final EList featureValue = ((EList) _eGet);
    featureValue.add(index, element);
  }
  
  @Override
  protected void doChangeIndex(final EObject container, final EStructuralFeature feature, final Object element, final int index) {
    Object _eGet = container.eGet(feature);
    final EList featureValue = ((EList) _eGet);
    featureValue.move(index, element);
  }
  
  @Override
  protected void doRemove(final EObject element) {
    EcoreUtil.remove(element);
  }
  
  @Override
  protected void doRemove(final EObject container, final EStructuralFeature feature, final Object element) {
    boolean _isMany = feature.isMany();
    if (_isMany) {
      Object _eGet = container.eGet(feature);
      final EList featureValueList = ((EList) _eGet);
      featureValueList.remove(element);
    } else {
      container.eUnset(feature);
    }
  }
  
  @Override
  protected void doRemove(final EObject container, final EStructuralFeature feature, final int index) {
    Object _eGet = container.eGet(feature);
    final EList featureValueList = ((EList) _eGet);
    featureValueList.remove(index);
  }
}
